@@ -1,4 +1,4 @@
 #ifndef  RF_STACK_INCOMING_H
 #define  RF_STACK_INCOMING_H
-void processDefaultPredict();
+void processDefaultPredict(void);
 #endif
